@@ -1,28 +1,37 @@
 """Run the generation process"""
 
+import os.path
 import sys
 import textwrap
 from typing import Any, Optional
 
+from . import config
 from . import dfd_dot_templates as TMPL
-from . import dot, model, parser, scanner, config
+from . import dot, model, parser, scanner, dependency_checker
 
 
 def build(provenance: model.SourceLine, dfd_src: str, output_path: str,
-          percent_zoom: int, bgcolor: str, format: str, debug: bool,
+          options: model.Options,
           snippet_by_name: model.SnippetByName = None) -> None:
     """Take a DFD source and build the final image or document"""
-    lines = scanner.scan(provenance, dfd_src, snippet_by_name, debug)
-    statements = parser.parse(lines, debug)
+    lines = scanner.scan(provenance, dfd_src, snippet_by_name, options.debug)
+    statements, dependencies = parser.parse(lines, options.debug)
+    if dependencies and not options.no_check_dependencies:
+        dependency_checker.check(dependencies, snippet_by_name, options)
     items_by_name = parser.check(statements)
     statements = filter_statements(statements)
     statements, graph_options = handle_options(statements)
 
+    if options.no_graph_title:
+        title = None
+    else:
+        title = os.path.splitext(output_path)[0]
+
     gen = Generator(graph_options)
-    text = generate_dot(gen, statements, items_by_name)
-    if debug:
+    text = generate_dot(gen, title, statements, items_by_name)
+    if options.debug:
         print(text)
-    dot.generate_image(graph_options, text, output_path, format)
+    dot.generate_image(graph_options, text, output_path, options.format)
 
 
 def wrap(text: str, cols: int) -> str:
@@ -56,6 +65,10 @@ class Generator:
                     fc = '"#eeeeee"'
                 line = (f'"{item.name}" [shape={shape} label="{text}" '
                         f'fillcolor={fc} style=filled {attrs}]')
+            case model.CONTROL:
+                fc = '"#eeeeee"'
+                line = (f'"{item.name}" [shape=ellipse label="{text}" '
+                        f'fillcolor={fc} style="filled,dashed" {attrs}]')
             case model.ENTITY:
                 line = (f'"{item.name}" [shape=rectangle label="{text}" '
                         f'{attrs}]')
@@ -82,7 +95,6 @@ class Generator:
         return d
 
     def generate_star(self, text: str) -> str:
-        print(text)
         text = wrap(text, self.graph_options.item_text_width)
         star_name = f'__star_{self.star_nr}__'
         line = f'"{star_name}" [shape=none label="{text}" {TMPL.DOT_FONT_EDGE}]'
@@ -122,6 +134,8 @@ class Generator:
                     attrs += ' dir=back'
             case model.BFLOW:
                 attrs += ' dir=both'
+            case model.CFLOW:
+                attrs += ' arrowhead=normalnormal'
             case model.UFLOW:
                 attrs += ' dir=none'
             case model.SIGNAL:
@@ -141,11 +155,14 @@ class Generator:
     def generate_style(self, style: model.Style) -> None:
         pass
 
-    def generate_dot_text(self) -> str:
+    def generate_dot_text(self, title: str) -> str:
         graph_params = []
         if self.graph_options.is_context:
             graph_params.append(TMPL.GRAPH_PARAMS_CONTEXT_DIAGRAM)
 
+        if title:
+            graph_params.append(TMPL.DOT_GRAPH_TITLE.format(title=title))
+
         if self.graph_options.is_vertical:
             graph_params.append('rankdir=TB')
         else:
@@ -153,6 +170,7 @@ class Generator:
 
         block = '\n'.join(self.lines).replace('\n', '\n  ')
         text = TMPL.DOT.format(
+            title=title,
             block=block,
             graph_params='\n  '.join(graph_params),
         ).replace('\n  \n', '\n\n')
@@ -160,7 +178,7 @@ class Generator:
         return text
 
 
-def generate_dot(gen: Generator, statements: model.Statements,
+def generate_dot(gen: Generator, title: str, statements: model.Statements,
                  items_by_name: dict[str, model.Item]) -> str:
     """Iterate over statements and generate a dot source file"""
     def get_item(name: str) -> Optional[model.Item]:
@@ -179,7 +197,7 @@ def generate_dot(gen: Generator, statements: model.Statements,
             case model.Style() as style:
                 gen.generate_style(style)
 
-    return gen.generate_dot_text()
+    return gen.generate_dot_text(title)
 
 
 def filter_statements(statements: model.Statements) -> model.Statements:
