@@ -37,6 +37,7 @@ class SourceLine(Base):
     raw_text: str
     parent: SourceLine  # https://stackoverflow.com/a/62521947
     line_nr: int
+    is_container: bool = False
 
 
 @dataclass
@@ -74,6 +75,7 @@ class Style(Statement):
 STYLE   = 'style'
 
 PROCESS = 'process'
+CONTROL = 'control'
 ENTITY  = 'entity'
 STORE   = 'store'
 CHANNEL = 'channel'
@@ -81,12 +83,13 @@ NONE    = 'none'
 
 FLOW    = 'flow'
 BFLOW   = 'bflow'
+CFLOW   = 'cflow'
 UFLOW   = 'uflow'
 SIGNAL  = 'signal'
 
 
 @dataclass
-class GraphOptions():
+class GraphOptions:
     is_vertical: bool = False
     is_context: bool = False
     item_text_width = config.DEFAULT_ITEM_TEXT_WIDTH
@@ -105,7 +108,14 @@ def mk_err_prefix_from(src: SourceLine) -> str:
         if src.line_nr is None:
             stack += [f'  {pack(src.raw_text)}']
         else:
-            stack += [f'  line {src.line_nr+1}: {pack(src.raw_text)}']
+            if src.parent and src.parent.is_container:
+                nr = src.parent.line_nr+1
+                delta = src.line_nr+1
+                final = nr + delta
+                stack += [f'  line {final}: {pack(src.raw_text)}']
+            else:
+                nr = src.line_nr+1
+                stack += [f'  line {nr}: {pack(src.raw_text)}']
         if src.parent:
             _add_to_stack(stack, src.parent)
 
@@ -124,3 +134,23 @@ Snippets = list[Snippet]
 SourceLines = list[SourceLine]
 Statements = list[Statement]
 SnippetByName = dict[str, Snippet]
+
+
+@dataclass
+class Options:
+    format: str
+    percent_zoom: int
+    background_color: str
+    no_graph_title: bool
+    no_check_dependencies: bool
+    debug: bool
+
+
+@dataclass
+class GraphDependency:
+    to_graph: str
+    to_item: str
+    to_type: str
+    source: SourceLine
+
+GraphDependencies = list[GraphDependency]
\ No newline at end of file
