@@ -1,8 +1,10 @@
 """Parse DFD source text in our DSL"""
 
+import os.path
 import re
 from typing import Callable, Tuple
 
+from . import dfd_dot_templates as TMPL
 from . import model
 
 
@@ -39,11 +41,17 @@ def check(statements: model.Statements) -> dict[str, model.Item]:
         for point in conn.src, conn.dst:
             if point == "*":
                 nb_stars += 1
-            if point == "*" or point in items_by_name:
-                continue
-            raise model.DfdException(
-                f'{error_prefix}Connection "{conn.type}" links to "{point}", '
-                f'which is not defined')
+            if point != "*":
+                if point not in items_by_name:
+                    raise model.DfdException(
+                        f'{error_prefix}Connection "{conn.type}" links to "{point}", '
+                        f'which is not defined')
+                if items_by_name[point].type == model.CONTROL and conn.type != model.SIGNAL:
+                    raise model.DfdException(
+                        f'{error_prefix}Connection to {model.CONTROL} "{point}" is '
+                        f'of type "{conn.type}", however only connections of type '
+                        f'"{model.SIGNAL}" are allowed')
+
         if nb_stars == 2:
             raise model.DfdException(
                 f'{error_prefix}Connection "{conn.type}" may not link to two '
@@ -53,10 +61,11 @@ def check(statements: model.Statements) -> dict[str, model.Item]:
 
 
 def parse(source_lines: model.SourceLines, debug: bool = False,
-         ) -> model.Statements:
+         ) -> tuple[model.Statements, model.GraphDependencies]:
     """Parse the DFD source text as list of statements"""
 
     statements: model.Statements = []
+    dependencies: model.GraphDependencies = []
 
     for n, source in enumerate(source_lines):
         src_line = source.text
@@ -74,17 +83,20 @@ def parse(source_lines: model.SourceLines, debug: bool = False,
         f: Callable[[model.SourceLine], model.Statement] = {
             model.STYLE: parse_style,
             model.PROCESS: parse_process,
+            model.CONTROL: parse_control,
             model.ENTITY: parse_entity,
             model.STORE: parse_store,
             model.NONE: parse_none,
             model.CHANNEL: parse_channel,
 
             model.FLOW: parse_flow,
+            model.CFLOW: parse_cflow,
             model.BFLOW: parse_bflow,
             model.UFLOW: parse_uflow,
             model.SIGNAL: parse_signal,
 
             model.FLOW+'?': parse_flow_q,
+            model.CFLOW+'?': parse_cflow_q,
             model.BFLOW+'?': parse_bflow_q,
             model.UFLOW+'?': parse_uflow_q,
             model.SIGNAL+'?': parse_signal_q,
@@ -106,6 +118,11 @@ def parse(source_lines: model.SourceLines, debug: bool = False,
         except model.DfdException as e:
             raise model.DfdException(f'{error_prefix}{e}"')
 
+        match statement:
+            case model.Item() as item:
+                parse_item_external(item, dependencies)
+                item.text = item.text or item.name
+
         match statement:
             case model.Drawable() as drawable:
                 parse_drawable_attrs(drawable)
@@ -114,19 +131,16 @@ def parse(source_lines: model.SourceLines, debug: bool = False,
 
     if debug:
         for s in statements: print(model.repr(s))
-    return statements
+    return statements, dependencies
 
 
-def split_args(dfd_line: str, n: int, last_is_optional: bool=False,
-               make_last_from_previous: bool=False) -> list[str]:
+def split_args(dfd_line: str, n: int, last_is_optional: bool=False) -> list[str]:
     """Split DFD line into n (possibly n-1) tokens"""
 
     terms = dfd_line.split(maxsplit=n)
     if len(terms)-1 == n-1 and last_is_optional:
-        if make_last_from_previous:
-            terms.append(terms[-1])
-        else:
-            terms.append(None)
+        terms.append(None)
+
     if len(terms)-1 != n:
         if not last_is_optional:
             raise model.DfdException(f'Expected {n} argument(s)')
@@ -152,35 +166,42 @@ def parse_style(source: model.SourceLine) -> model.Statement:
 
 def parse_process(source: model.SourceLine) -> model.Statement:
     """Parse process statement"""
-    name, text = split_args(source.text, 2, True, True)
+    name, text = split_args(source.text, 2, True)
     name, hidable = parse_item_name(name)
     return model.Item(source, model.PROCESS, text, "", name, hidable)
 
 
+def parse_control(source: model.SourceLine) -> model.Statement:
+    """Parse control statement"""
+    name, text = split_args(source.text, 2, True)
+    name, hidable = parse_item_name(name)
+    return model.Item(source, model.CONTROL, text, "", name, hidable)
+
+
 def parse_entity(source: model.SourceLine) -> model.Statement:
     """Parse entity statement"""
-    name, text = split_args(source.text, 2, True, True)
+    name, text = split_args(source.text, 2, True)
     name, hidable = parse_item_name(name)
     return model.Item(source, model.ENTITY, text, "", name, hidable)
 
 
 def parse_store(source: model.SourceLine) -> model.Statement:
     """Parse store statement"""
-    name, text = split_args(source.text, 2, True, True)
+    name, text = split_args(source.text, 2, True)
     name, hidable = parse_item_name(name)
     return model.Item(source, model.STORE, text, "", name, hidable)
 
 
 def parse_none(source: model.SourceLine) -> model.Statement:
     """Parse none statement"""
-    name, text = split_args(source.text, 2, True, True)
+    name, text = split_args(source.text, 2, True)
     name, hidable = parse_item_name(name)
     return model.Item(source, model.NONE, text, "", name, hidable)
 
 
 def parse_channel(source: model.SourceLine) -> model.Statement:
     """Parse channel statement"""
-    name, text = split_args(source.text, 2, True, True)
+    name, text = split_args(source.text, 2, True)
     name, hidable = parse_item_name(name)
     return model.Item(source, model.CHANNEL, text, "", name, hidable)
 
@@ -197,6 +218,12 @@ def parse_flow_r(source: model.SourceLine) -> model.Statement:
     return model.Connection(source, model.FLOW, text, "", src, dst, True)
 
 
+def parse_cflow(source: model.SourceLine) -> model.Statement:
+    """Parse directional flow statement"""
+    src, dst, text = split_args(source.text, 3, True)
+    return model.Connection(source, model.CFLOW, text, "", src, dst)
+
+
 def parse_bflow(source: model.SourceLine) -> model.Statement:
     """Parse bidirectional flow statement"""
     src, dst, text = split_args(source.text, 3, True)
@@ -234,6 +261,13 @@ def parse_flow_r_q(source: model.SourceLine) -> model.Statement:
                             relaxed=True)
 
 
+def parse_cflow_q(source: model.SourceLine) -> model.Statement:
+    """Parse directional flow statement"""
+    src, dst, text = split_args(source.text, 3, True)
+    return model.Connection(source, model.CFLOW, text, "", src, dst,
+                            relaxed=True)
+
+
 def parse_bflow_q(source: model.SourceLine) -> model.Statement:
     """Parse bidirectional flow statement"""
     src, dst, text = split_args(source.text, 3, True)
@@ -286,6 +320,15 @@ def apply_syntactic_sugars(src_line: str) -> str:
         parts = src_line.split(maxsplit=3)
         new_line = fmt('flow.r'+q, parts)#, swap=True)
 
+    if re.fullmatch(r'-+>>[?]?', op):
+        q = '?' if op.endswith('?') else ''
+        parts = src_line.split(maxsplit=3)
+        new_line = fmt('cflow'+q, parts)
+    elif re.fullmatch(r'<<-+[?]?', op):
+        q = '?' if op.endswith('?') else ''
+        parts = src_line.split(maxsplit=3)
+        new_line = fmt('cflow.r'+q, parts)#, swap=True)
+
     elif re.fullmatch(r'<-+>[?]?', op):
         q = '?' if op.endswith('?') else ''
         parts = src_line.split(maxsplit=3)
@@ -320,3 +363,27 @@ def parse_drawable_attrs(drawable: model.Drawable) -> None:
         match drawable:
             case model.Item() as item:
                 item.text = item.text or item.name
+
+
+def parse_item_external(item: model.Item, dependencies: model.GraphDependencies) -> None:
+    parts = item.name.split(':', 1)
+    if len(parts) > 1:
+        item.attrs = TMPL.ITEM_EXTERNAL_ATTRS
+        if parts[-1]:
+            item.name = parts[-1]
+        else:
+            item.name = parts[-2]
+
+        if item.name.startswith('#'):
+            item.name = item.name[1:]
+        else:
+            item.name = os.path.splitext(item.name)[0]
+
+        if not item.text:
+            item.text = item.name
+
+        dependency = model.GraphDependency(parts[0],
+                                           parts[1] or None,
+                                           item.type,
+                                           item.source)
+        dependencies.append(dependency)
